/*
 * ------------------------------------------------------------------------------
 * Just two functions that set and get the rotation for the actuators in 5 linkage
 * paralel arm configuration based on X & Y input. The same fucntions drive a haptic drawing
 * prototype called Scribble.: http://felixros.com/scribble.html
 * 
 * V 1.0 - tested on Arduino DUE
 * Written and modeified by Felix Ros 2018
 * 
 * "THE BEER-WARE LICENSE" (Revision 42):
 * Felix Ros wrote this file.  As long as you retain this notice you
 * can do whatever you want with this stuff. If we meet some day, and you think
 * this stuff is worth it, you can buy me a beer in return.   
 * (license by Poul-Henning Kamp)
 * ------------------------------------------------------------------------------
 */

int leftActuator, rightActuator; //angle to write to the actuator in deg, change to floats if you desire more acuracy
int posX, posY; //the coordinates of the location of the pointer

/*
 * ------------------------------------------------------------------------------
 * set your hardware properties
 * ------------------------------------------------------------------------------
 */

//set the resolution of your input 
int posStepsX = 2000;
int posStepsY = 1000;

// values are in mm  
#define L1 73 // length motor arm, see SAP picture (left and right are the same)
#define L2 95 // length extention arm, see SAP picture (left and right are the same)

#define rangeX 250 // maximum range in X direction for the point to move (from left to right, 0 - maxVal)
#define rangeY 165 // maximum range in Y direction for the point to move (from 0 to maximim reach while staying centered)

#define originL 90 //offset distance from most minumim X value to actuator center position
#define originR 145 //offset distance from most minumim X value to actuator center position, the distance between the two motors is in this case 55mm

/*
 * ------------------------------------------------------------------------------
 * forward kinematics
 * ------------------------------------------------------------------------------
 */
double return_angle(double a, double b, double c) {
  // cosine rule for angle between c and a
  return acos((a * a + c * c - b * b) / (2 * a * c));
}
void set_XY(double Tx, double Ty) //input your X and Y value
{
  // some vals we need but don't want to save for long
  double dx, dy, c, a1, a2, Hx, Hy;

  //map input resolution to range of your configuration in the real world
  int realX = map(Tx, 0, posStepsX, 0, rangeX); //swap if mapping if inversed
  int realY = map(Ty, posStepsX, 0, 0, rangeY); //swap if mapping if inversed

  // calc angle for left actuator
  // cartesian dx/dy
  dx = realX - originL; //include offset
  dy = realY;

  // polar length (c) and angle (a1)
  c = sqrt(dx * dx + dy * dy);
  a1 = atan2(dy, dx);
  a2 = return_angle(L1, L2, c);

  leftActuator = floor(((M_PI - (a2 + a1)) * 4068) / 71); //final angle and convert from rad to deg

  // calc angle for right actuator
  dx = realX - originR; //include offset
  dy = realY;

  c = sqrt(dx * dx + dy * dy);
  a1 = atan2(dy, dx);
  a2 = return_angle(L1, L2, c);

  rightActuator = floor(((a1 - a2) * 4068) / 71); //final angle and convert from rad to deg
}

/*
 * ------------------------------------------------------------------------------
 * inverse kinematics
 * ------------------------------------------------------------------------------
 */
void get_XY(float degL, float degR) { //input the actuator's angle

  // some vals we need but don't want to save for long
  double radL, radR, X1l, Y1l, X1r, Y1r, X1m, Y1m, d, p, Xget, Yget, Xget1, Yget1, Xget2, Yget2;

  // turn deg into rad for calculations
  radL = (degL * 71) / 4068;
  radR = (degR * 71) / 4068;

  // position left joint
  X1l = originL - (sin(radL) * L1);
  Y1l = (cos(radL) * L1);

  // position right joint
  X1r = originR + (sin(radR) * L1);
  Y1r = (cos(radR) * L1);

  d = sqrt(sq(X1r - X1l) + sq(Y1r - Y1l)) / 2; //calc dist diagonal line between joints and devide in 2
  p = sqrt(L2 * L2 - d * d);//percenucular line straigt through d

  // get intersection position of d and p
  X1m = (X1l + d * (X1r - X1l)) / (d * 2);
  Y1m = (Y1l + d * (Y1r - Y1l)) / (d * 2);

  // get intersections
  Xget1 =  (X1m + p * (Y1r - Y1l)) / (d * 2);
  Xget2 =  (X1m - p * (Y1r - Y1l)) / (d * 2);
  Yget1 =  (Y1m + p * (X1r - X1l)) / (d * 2);
  Yget2 =  (Y1m - p * (X1r - X1l)) / (d * 2);

  // final real world coordinates
  Xget = X1l + Xget2 + X1m;
  Yget = Y1r + Yget1 - Y1m;

  // translate from mm to input resolution
  posX = map(Xget, 0, rangeX, 0, posStepsX);
  posY = map(Yget, rangeY, 0, 0, posStepsY);

  posX = constrain(posX, 0, posStepsX);
  posY = constrain(posY, 0, posStepsY);

}





